#include "DataSaver.h"
#include "Info.h"
#include "OF_HEPEVT.h"
#include "OF_LHEF.h"
#include "OutputFormat.h"
#include "Process.h"
#ifdef HEPMC
#include "OF_HepMC.h"
#endif

using namespace SANC;

DataSaver::DataSaver(Info &_info, Process &_proc) : info(_info), proc(_proc) {
  if (info.PrintLHE == 1) {
    writer = new OF_LHEF(info, proc);
  } else if (info.PrintHEPEVT == 1) {
    writer = new OF_HEPEVT(info, proc);
  }
#ifdef HEPMC
  else if (info.PrintHepMC == 1) {
    writer = new OF_HEPMC(info, proc);
  }
#endif
  ready = (writer) ? true : false;
}

void DataSaver::start(std::string dirName, std::string fileName) {
  if (writer) {
    writer->start(dirName, fileName);
  }
}

void DataSaver::writeEvent() {
  if (writer) {
    writer->writeEvent();
  }
}

void DataSaver::finish() {
  if (writer) {
    writer->finish();
  }
}