#include "OF_HepMC.h"
#include "HepMC3/WriterAscii.h"
#include "HepMC3/WriterAsciiHepMC2.h"
#include "HepMC3/WriterHEPEVT.h"
#include "Info.h"
#include "Process.h"

using namespace SANC;

OF_HEPMC::OF_HEPMC(Info &_info, Process &_proc) : OutputFormat(_info, _proc) {
  // hepmc = new HepMC3::GenEvent();
  xsec = std::make_shared<HepMC3::GenCrossSection>();
}

void OF_HEPMC::start(std::string _dirName, std::string _fileName) {
  dirName = _dirName;
  fileName = _fileName;
  if (info.HepMCFormat == "Ascii") {
    output = new HepMC3::WriterAscii(dirName + "/" + fileName + ".hepmc");
  } else if (info.HepMCFormat == "HepMC2") {
    output = new HepMC3::WriterAsciiHepMC2(dirName + "/" + fileName + ".hepmc");
  } else if (info.HepMCFormat == "HEPEVT") {
    output = new HepMC3::WriterHEPEVT(dirName + "/" + fileName + ".hepevt");
  } else {
    output = new HepMC3::WriterAscii(dirName + "/" + fileName + ".hepmc");
  }
  nEvents = 0;
}

void OF_HEPMC::writeEvent() {
  HepMC3::GenEvent hepmc;
  hepmc.set_event_number(nEvents);
  hepmc.set_units(HepMC3::Units::GEV, HepMC3::Units::MM);
  // std::vector<HepMC3::GenParticlePtr> hepevt_particles;
  hepevt_particles.clear();
  hepevt_particles.reserve(proc.nup + 1);
  //  proc.nup+1 particle, where zero particle is system
  for (int i = 0; i <= proc.nup; ++i) {
    hepevt_particles.push_back(std::make_shared<HepMC3::GenParticle>(
        HepMC3::FourVector(proc.event.P_lab(i).px(), proc.event.P_lab(i).py(),
                           proc.event.P_lab(i).pz(), proc.event.P_lab(i).e()),
        proc.idup[i], proc.event.particles[i].statusHepMC));
    hepevt_particles[i]->set_generated_mass(proc.event.P_lab(i).m());
  }

  // std::vector<HepMC3::GenVertexPtr> vertex_cache;
  vertex_cache.clear();
  for (int i = 1; i <= proc.nup; ++i) {
    std::vector<int> mothers = proc.event.motherList(i);
    if (mothers.size()) {
      HepMC3::GenVertexPtr prod_vtx = hepevt_particles[mothers[0]]->end_vertex();
      if (!prod_vtx) {
        prod_vtx = std::make_shared<HepMC3::GenVertex>();
        vertex_cache.push_back(prod_vtx);
        for (unsigned int j = 0; j < mothers.size(); ++j)
          prod_vtx->add_particle_in(hepevt_particles[mothers[j]]);
      }
      // HepMC3::FourVector prod_pos( pyev[i].xProd(), pyev[i].yProd(),pyev[i].zProd(),
      //   pyev[i].tProd() );

      // Update vertex position if necessary.
      // if (!prod_pos.is_zero() && prod_vtx->position().is_zero())
      //   prod_vtx->set_position( prod_pos );
      prod_vtx->add_particle_out(hepevt_particles[i]);
    }
  }

  hepmc.reserve(hepevt_particles.size(), vertex_cache.size());
  // std::cout<< "hepevt_particles.size()="<<hepevt_particles.size() <<"
  // vertex_cache.size()="<<vertex_cache.size()<<std::endl; std::vector<HepMC3::GenParticlePtr>
  // beam_particles;
  beam_particles.clear();
  beam_particles.push_back(hepevt_particles[1]);
  beam_particles.push_back(hepevt_particles[2]);
  hepmc.add_tree(beam_particles);

  hepmc.weights().push_back(proc.event.weight());

  // Store cross-section in pb
  // hepmc.set_cross_section(xsec);
  // xsec->set_cross_section(info.Sigma, info.SigmaErr);
  output->write_event(hepmc);
  nEvents++;
}

void OF_HEPMC::finish() {
  output->close();
  delete output;
}